@@ -41,6 +41,7 @@ class TestPort(network_fakes.TestNetworkV2):
             'binding_vif_details',
             'binding_vif_type',
             'binding_vnic_type',
+            'description',
             'device_id',
             'device_owner',
             'dns_assignment',
@@ -65,6 +66,7 @@ class TestPort(network_fakes.TestNetworkV2):
             utils.format_dict(fake_port.binding_vif_details),
             fake_port.binding_vif_type,
             fake_port.binding_vnic_type,
+            fake_port.description,
             fake_port.device_id,
             fake_port.device_owner,
             utils.format_list_of_dicts(fake_port.dns_assignment),
@@ -130,6 +132,7 @@ class TestCreatePort(TestPort):
             '--mac-address', 'aa:aa:aa:aa:aa:aa',
             '--fixed-ip', 'subnet=%s,ip-address=10.0.0.2'
             % self.fake_subnet.id,
+            '--description', self._port.description,
             '--device', 'deviceid',
             '--device-owner', 'fakeowner',
             '--disable',
@@ -137,6 +140,7 @@ class TestCreatePort(TestPort):
             '--binding-profile', 'foo=bar',
             '--binding-profile', 'foo2=bar2',
             '--network', self._port.network_id,
+            '--dns-name', '8.8.8.8',
             'test-port',
 
         ]
@@ -146,12 +150,14 @@ class TestCreatePort(TestPort):
                 'fixed_ip',
                 [{'subnet': self.fake_subnet.id, 'ip-address': '10.0.0.2'}]
             ),
+            ('description', self._port.description),
             ('device', 'deviceid'),
             ('device_owner', 'fakeowner'),
             ('disable', True),
             ('vnic_type', 'macvtap'),
             ('binding_profile', {'foo': 'bar', 'foo2': 'bar2'}),
             ('network', self._port.network_id),
+            ('dns_name', '8.8.8.8'),
             ('name', 'test-port'),
 
         ]
@@ -163,12 +169,14 @@ class TestCreatePort(TestPort):
             'mac_address': 'aa:aa:aa:aa:aa:aa',
             'fixed_ips': [{'subnet_id': self.fake_subnet.id,
                            'ip_address': '10.0.0.2'}],
+            'description': self._port.description,
             'device_id': 'deviceid',
             'device_owner': 'fakeowner',
             'admin_state_up': False,
             'binding:vnic_type': 'macvtap',
             'binding:profile': {'foo': 'bar', 'foo2': 'bar2'},
             'network_id': self._port.network_id,
+            'dns_name': '8.8.8.8',
             'name': 'test-port',
         })
 
@@ -228,6 +236,169 @@ class TestCreatePort(TestPort):
         self.assertEqual(ref_columns, columns)
         self.assertEqual(ref_data, data)
 
+    def test_create_with_security_group(self):
+        secgroup = network_fakes.FakeSecurityGroup.create_one_security_group()
+        self.network.find_security_group = mock.Mock(return_value=secgroup)
+        arglist = [
+            '--network', self._port.network_id,
+            '--security-group', secgroup.id,
+            'test-port',
+        ]
+
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('security_groups', [secgroup.id]),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'security_groups': [secgroup.id],
+            'name': 'test-port',
+        })
+
+        ref_columns, ref_data = self._get_common_cols_data(self._port)
+        self.assertEqual(ref_columns, columns)
+        self.assertEqual(ref_data, data)
+
+    def test_create_port_with_dns_name(self):
+        arglist = [
+            '--network', self._port.network_id,
+            '--dns-name', '8.8.8.8',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('dns_name', '8.8.8.8'),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'dns_name': '8.8.8.8',
+            'name': 'test-port',
+        })
+
+        ref_columns, ref_data = self._get_common_cols_data(self._port)
+        self.assertEqual(ref_columns, columns)
+        self.assertEqual(ref_data, data)
+
+    def test_create_with_security_groups(self):
+        sg_1 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        sg_2 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        self.network.find_security_group = mock.Mock(side_effect=[sg_1, sg_2])
+        arglist = [
+            '--network', self._port.network_id,
+            '--security-group', sg_1.id,
+            '--security-group', sg_2.id,
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('security_groups', [sg_1.id, sg_2.id]),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'security_groups': [sg_1.id, sg_2.id],
+            'name': 'test-port',
+        })
+
+        ref_columns, ref_data = self._get_common_cols_data(self._port)
+        self.assertEqual(ref_columns, columns)
+        self.assertEqual(ref_data, data)
+
+    def test_create_with_no_secuirty_groups(self):
+        arglist = [
+            '--network', self._port.network_id,
+            '--no-security-group',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id),
+            ('enable', True),
+            ('no_security_group', True),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'security_groups': [],
+            'name': 'test-port',
+        })
+
+        ref_columns, ref_data = self._get_common_cols_data(self._port)
+        self.assertEqual(ref_columns, columns)
+        self.assertEqual(ref_data, data)
+
+    def test_create_port_security_enabled(self):
+        arglist = [
+            '--network', self._port.network_id,
+            '--enable-port-security',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('enable_port_security', True),
+            ('name', 'test-port'),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'port_security_enabled': True,
+            'name': 'test-port',
+        })
+
+    def test_create_port_security_disabled(self):
+        arglist = [
+            '--network', self._port.network_id,
+            '--disable-port-security',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('disable_port_security', True),
+            ('name', 'test-port'),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'port_security_enabled': False,
+            'name': 'test-port',
+        })
+
 
 class TestDeletePort(TestPort):
 
@@ -317,6 +488,17 @@ class TestListPort(TestPort):
         'Name',
         'MAC Address',
         'Fixed IP Addresses',
+        'Status',
+    )
+
+    columns_long = (
+        'ID',
+        'Name',
+        'MAC Address',
+        'Fixed IP Addresses',
+        'Status',
+        'Security Groups',
+        'Device Owner',
     )
 
     data = []
@@ -326,6 +508,19 @@ class TestListPort(TestPort):
             prt.name,
             prt.mac_address,
             utils.format_list_of_dicts(prt.fixed_ips),
+            prt.status,
+        ))
+
+    data_long = []
+    for prt in _ports:
+        data_long.append((
+            prt.id,
+            prt.name,
+            prt.mac_address,
+            utils.format_list_of_dicts(prt.fixed_ips),
+            prt.status,
+            utils.format_list(prt.security_groups),
+            prt.device_owner,
         ))
 
     def setUp(self):
@@ -419,12 +614,14 @@ class TestListPort(TestPort):
             '--device-owner', self._ports[0].device_owner,
             '--router', 'fake-router-name',
             '--network', 'fake-network-name',
+            '--mac-address', self._ports[0].mac_address,
         ]
 
         verifylist = [
             ('device_owner', self._ports[0].device_owner),
             ('router', 'fake-router-name'),
-            ('network', 'fake-network-name')
+            ('network', 'fake-network-name'),
+            ('mac_address', self._ports[0].mac_address)
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -434,11 +631,48 @@ class TestListPort(TestPort):
         self.network.ports.assert_called_once_with(**{
             'device_owner': self._ports[0].device_owner,
             'device_id': 'fake-router-id',
-            'network_id': 'fake-network-id'
+            'network_id': 'fake-network-id',
+            'mac_address': self._ports[0].mac_address
         })
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_port_list_mac_address_opt(self):
+        arglist = [
+            '--mac-address', self._ports[0].mac_address,
+        ]
+
+        verifylist = [
+            ('mac_address', self._ports[0].mac_address)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(**{
+            'mac_address': self._ports[0].mac_address
+        })
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_list_port_with_long(self):
+        arglist = [
+            '--long',
+        ]
+
+        verifylist = [
+            ('long', True),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with()
+        self.assertEqual(self.columns_long, columns)
+        self.assertEqual(self.data_long, list(data))
+
 
 class TestSetPort(TestPort):
 
@@ -473,6 +707,25 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
+    def test_set_dns_name(self):
+        arglist = [
+            '--dns-name', '8.8.8.8',
+            self._port.name,
+        ]
+        verifylist = [
+            ('dns_name', '8.8.8.8'),
+            ('port', self._port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'dns_name': '8.8.8.8',
+        }
+        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        self.assertIsNone(result)
+
     def test_append_fixed_ip(self):
         _testport = network_fakes.FakePort.create_one_port(
             {'fixed_ips': [{'ip_address': '0.0.0.1'}]})
@@ -565,6 +818,7 @@ class TestSetPort(TestPort):
 
     def test_set_that(self):
         arglist = [
+            '--description', 'newDescription',
             '--enable',
             '--vnic-type', 'macvtap',
             '--binding-profile', 'foo=bar',
@@ -573,6 +827,7 @@ class TestSetPort(TestPort):
             self._port.name,
         ]
         verifylist = [
+            ('description', 'newDescription'),
             ('enable', True),
             ('vnic_type', 'macvtap'),
             ('binding_profile', {'foo': 'bar'}),
@@ -589,6 +844,7 @@ class TestSetPort(TestPort):
             'binding:vnic_type': 'macvtap',
             'binding:profile': {'foo': 'bar'},
             'binding:host_id': 'binding-host-id-xxxx',
+            'description': 'newDescription',
             'name': 'newName',
         }
         self.network.update_port.assert_called_once_with(self._port, **attrs)
@@ -651,6 +907,131 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
+    def test_set_security_group(self):
+        sg = network_fakes.FakeSecurityGroup.create_one_security_group()
+        self.network.find_security_group = mock.Mock(return_value=sg)
+        arglist = [
+            '--security-group', sg.id,
+            self._port.name,
+        ]
+        verifylist = [
+            ('security_groups', [sg.id]),
+            ('port', self._port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'security_groups': [sg.id],
+        }
+        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        self.assertIsNone(result)
+
+    def test_append_security_group(self):
+        sg_1 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        sg_2 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        sg_3 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        self.network.find_security_group = mock.Mock(side_effect=[sg_2, sg_3])
+        _testport = network_fakes.FakePort.create_one_port(
+            {'security_groups': [sg_1.id]})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--security-group', sg_2.id,
+            '--security-group', sg_3.id,
+            _testport.name,
+        ]
+        verifylist = [
+            ('security_groups', [sg_2.id, sg_3.id]),
+            ('port', _testport.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'security_groups': [sg_1.id, sg_2.id, sg_3.id],
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_no_security_groups(self):
+        arglist = [
+            '--no-security-group',
+            self._port.name,
+        ]
+        verifylist = [
+            ('no_security_group', True),
+            ('port', self._port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'security_groups': [],
+        }
+        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        self.assertIsNone(result)
+
+    def test_overwrite_security_group(self):
+        sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        _testport = network_fakes.FakePort.create_one_port(
+            {'security_groups': [sg1.id]})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        self.network.find_security_group = mock.Mock(return_value=sg2)
+        arglist = [
+            '--security-group', sg2.id,
+            '--no-security-group',
+            _testport.name,
+        ]
+        verifylist = [
+            ('security_groups', [sg2.id]),
+            ('no_security_group', True)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'security_groups': [sg2.id],
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_port_security_enabled(self):
+        arglist = [
+            '--enable-port-security',
+            self._port.id,
+        ]
+        verifylist = [
+            ('enable_port_security', True),
+            ('port', self._port.id,)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.network.update_port.assert_called_once_with(self._port, **{
+            'port_security_enabled': True,
+        })
+
+    def test_port_security_disabled(self):
+        arglist = [
+            '--disable-port-security',
+            self._port.id,
+        ]
+        verifylist = [
+            ('disable_port_security', True),
+            ('port', self._port.id,)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        self.network.update_port.assert_called_once_with(self._port, **{
+            'port_security_enabled': False,
+        })
+
 
 class TestShowPort(TestPort):
 
@@ -767,3 +1148,47 @@ class TestUnsetPort(TestPort):
         self.assertRaises(exceptions.CommandError,
                           self.cmd.take_action,
                           parsed_args)
+
+    def test_unset_security_group(self):
+        _fake_sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        _fake_sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'security_groups': [_fake_sg1.id, _fake_sg2.id]})
+        self.network.find_port = mock.Mock(return_value=_fake_port)
+        self.network.find_security_group = mock.Mock(return_value=_fake_sg2)
+        arglist = [
+            '--security-group', _fake_sg2.id,
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('security_groups', [_fake_sg2.id]),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'security_groups': [_fake_sg1.id]
+        }
+        self.network.update_port.assert_called_once_with(
+            _fake_port, **attrs)
+        self.assertIsNone(result)
+
+    def test_unset_port_security_group_not_existent(self):
+        _fake_sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        _fake_sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'security_groups': [_fake_sg1.id]})
+        self.network.find_security_group = mock.Mock(return_value=_fake_sg2)
+        arglist = [
+            '--security-group', _fake_sg2.id,
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('security_groups', [_fake_sg2.id]),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action,
+                          parsed_args)
