@@ -18,6 +18,7 @@ from osc_lib import exceptions
 from osc_lib import utils as osc_utils
 
 from openstackclient.network.v2 import router
+from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes_v3
 from openstackclient.tests.unit.network.v2 import fakes as network_fakes
 from openstackclient.tests.unit import utils as tests_utils
 
@@ -29,6 +30,7 @@ class TestRouter(network_fakes.TestNetworkV2):
 
         # Get a shortcut to the network client
         self.network = self.app.client_manager.network
+        self.projects_mock = self.app.client_manager.identity.projects
 
 
 class TestAddPortToRouter(TestRouter):
@@ -427,6 +429,97 @@ class TestListRouter(TestRouter):
         self.assertEqual(self.columns_long_no_az, columns)
         self.assertEqual(self.data_long_no_az, list(data))
 
+    def test_list_name(self):
+        test_name = "fakename"
+        arglist = [
+            '--name', test_name,
+        ]
+        verifylist = [
+            ('long', False),
+            ('name', test_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.routers.assert_called_once_with(
+            **{'name': test_name}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_router_list_enable(self):
+        arglist = [
+            '--enable',
+        ]
+        verifylist = [
+            ('long', False),
+            ('enable', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.routers.assert_called_once_with(
+            **{'admin_state_up': True}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_router_list_disable(self):
+        arglist = [
+            '--disable',
+        ]
+        verifylist = [
+            ('long', False),
+            ('disable', True)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.routers.assert_called_once_with(
+            **{'admin_state_up': False}
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_router_list_project(self):
+        project = identity_fakes_v3.FakeProject.create_one_project()
+        self.projects_mock.get.return_value = project
+        arglist = [
+            '--project', project.id,
+        ]
+        verifylist = [
+            ('project', project.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'tenant_id': project.id}
+
+        self.network.routers.assert_called_once_with(**filters)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_router_list_project_domain(self):
+        project = identity_fakes_v3.FakeProject.create_one_project()
+        self.projects_mock.get.return_value = project
+        arglist = [
+            '--project', project.id,
+            '--project-domain', project.domain_id,
+        ]
+        verifylist = [
+            ('project', project.id),
+            ('project_domain', project.domain_id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'tenant_id': project.id}
+
+        self.network.routers.assert_called_once_with(**filters)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestRemovePortFromRouter(TestRouter):
     '''Remove port from a Router '''
@@ -529,6 +622,7 @@ class TestSetRouter(TestRouter):
             '--enable',
             '--distributed',
             '--name', 'noob',
+            '--no-ha',
             '--description', 'router',
         ]
         verifylist = [
@@ -537,6 +631,7 @@ class TestSetRouter(TestRouter):
             ('distributed', True),
             ('name', 'noob'),
             ('description', 'router'),
+            ('no_ha', True),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -546,6 +641,7 @@ class TestSetRouter(TestRouter):
             'admin_state_up': True,
             'distributed': True,
             'name': 'noob',
+            'ha': False,
             'description': 'router',
         }
         self.network.update_router.assert_called_once_with(
@@ -557,11 +653,13 @@ class TestSetRouter(TestRouter):
             self._router.name,
             '--disable',
             '--centralized',
+            '--ha',
         ]
         verifylist = [
             ('router', self._router.name),
             ('disable', True),
             ('centralized', True),
+            ('ha', True),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -570,6 +668,7 @@ class TestSetRouter(TestRouter):
         attrs = {
             'admin_state_up': False,
             'distributed': False,
+            'ha': True,
         }
         self.network.update_router.assert_called_once_with(
             self._router, **attrs)
@@ -773,9 +872,9 @@ class TestUnsetRouter(TestRouter):
         super(TestUnsetRouter, self).setUp()
         self._testrouter = network_fakes.FakeRouter.create_one_router(
             {'routes': [{"destination": "192.168.101.1/24",
-                         "gateway": "172.24.4.3"},
+                         "nexthop": "172.24.4.3"},
                         {"destination": "192.168.101.2/24",
-                         "gateway": "172.24.4.3"}], })
+                         "nexthop": "172.24.4.3"}], })
         self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet()
         self.network.find_router = mock.Mock(return_value=self._testrouter)
         self.network.update_router = mock.Mock(return_value=None)
